#!/usr/bin/env python
# -*- coding: utf8 -*-

import os
from inupypi import abort, app
from pkgtools.pkg import SDist
from unipath import Path


class Package(object):
    filepath = None
    name = None
    author = 'unknown'
    current = None

def get_eggbaskets():
    path = Path(app.config.get('EGGBASKET_REPO', ''))
    if not path.exists() and not path.isdir():
        abort(500, "%s doesn't exist." % path)
    return os.listdir(path)

def get_package_path(eggbasket):
    path = Path(app.config.get('EGGBASKET_REPO', ''), eggbasket)

    if not path.exists() and not path.isdir():
        abort(500)
    return path

def get_packages(eggbasket):
    packages = []
    folders = [Path(folder) for folder in get_package_path(eggbasket).listdir()
            if folder.isdir() and folder.listdir() != []]

    for package in folders:
        p = Package()
        p.filepath = package
        p.current = get_current_package(eggbasket, package)
        p.eggbasket = eggbasket 
        try:
            p.author = SDist(p.current).metadata['PKG-INFO']['Author']
        except Exception:
            pass
        packages.append(p)
    return packages

def get_package_files(eggbasket, package):
    package_path = get_package_path(eggbasket)
    package = package_path.listdir()[[x.name.lower() for x in package_path.listdir()].index(package.lower())]

    package_dir = Path(package_path, package)
    files = []

    for package_file in package_dir.listdir():
        p = Package()
        p.filepath = package_file
        p.eggbasket = eggbasket 
        try:
            p.name = SDist(package_file).name
            p.author = SDist(package_file).metadata['PKG-INFO']['Author']
        except Exception:
            pass
        files.append(p)
    return sorted(files, reverse=True)

def get_current_package(eggbasket, package):
    package_dir = Path(get_package_path(eggbasket), package)
    contents = sorted(package_dir.listdir(), reverse=True)
    return contents[0] if contents else None

def get_metadata(eggbasket, package, filename):
    package_file = Path(get_package_path(eggbasket), package, filename)

    try:
        return SDist(package_file)
    except Exception:
        return None

def get_file(eggbasket, package, filename):
    package_file = Path(get_package_path(eggbasket), package, filename)

    if package_file.exists() and package_file.isfile():
        return package_file
    return False



##!/usr/bin/env python
## -*- coding: utf8 -*-
#
#import tempfile
#import unittest
#
#from inupypi.components.packages import *
#from inupypi.components.packages.manage import *
#from tests.packages import *
#from unipath import Path
#from werkzeug import FileStorage
#
#
#class Test_Manage(unittest.TestCase):
#    def setUp(self):
#        self.app = app.test_client()
#        self.workspace = Path(tempfile.mkdtemp())
#        self.tmp_workspace = Path(tempfile.mkdtemp())
#        self.packages = ['p1', 'p2', 'p3', 'p4']
#        self.files = sorted(['f1', 'f2', 'f3', 'f4'], reverse=True)
#
#        self.app.application.config['PACKAGE_PATH'] = self.workspace
#
#    def tearDown(self):
#        self.workspace.rmtree()
#        self.tmp_workspace.rmtree()
#
#    def test_upload_package(self):
#        assert get_packages() == []
#
#        env_create_package_files(self.tmp_workspace, self.packages, self.files)
#
#        for p in self.packages:
#            for f in self.files:
#                filename = u'%s.tar.gz' % f
#                stream = open(Path(self.tmp_workspace, p, filename))
#                fs = FileStorage(filename=filename, stream=stream)
#                upload_package(fs)
#        packages = [p.filepath for p in get_packages()]
#        created_packages = [Path(self.workspace, package)
#                for package in self.packages]
#        assert packages == created_packages
#
#        for p in self.packages:
#            files = sorted([Path(self.workspace, p, '%s.tar.gz' % f)
#                    for f in self.files])
#            package_files = sorted([pkg.filepath
#                for pkg in get_package_files(p)])
#            assert files == package_files



#!/usr/bin/env python
# -*- coding: utf8 -*-

import inupypi
import tempfile
import unittest

from tests.packages import *
from unipath import Path


class Test_Inupypi(unittest.TestCase):
    def setUp(self):
        self.app = inupypi.app.test_client()
        self.workspace = Path(tempfile.mkdtemp())
        self.packages = ['p1', 'p2', 'p3', 'p4']
        self.files = ['f1', 'f2', 'f3', 'f4']
        self.app.application.config['EGGBASKET_REPO'] = self.workspace

    def tearDown(self):
        self.workspace.rmtree()

    def test_app_with_missing_package_dir(self):
        self.app.application.config['EGGBASKET_REPO'] = Path(self.workspace, 'a')
        assert self.app.get('/').status_code == 500
        assert '500 - Internal Server Error' in self.app.get('/').data

    def test_app_without_packages(self):
        assert 'inetutils PyPI Server' in self.app.get('/').data
        assert 'Available Packages' not in self.app.get('/').data

    def test_app_with_package_folders(self):
        env_create_packages(self.workspace, self.packages)
        env_create_package_files(self.workspace, self.packages, self.files)
        assert 'inetutils PyPI Server' in self.app.get('/').data
        assert 'Available EggBaskets' in self.app.get('/').data

    def test_app_package(self):
        env_create_packages(self.workspace, self.packages)

        for p in self.packages:
            for f in self.files:
                page = self.app.get('/'+p+'/').data
                assert '404 - Page not found' not in page
                assert f not in page

        env_create_package_files(self.workspace, self.packages, self.files)

        for p in self.packages:
            for f in self.files:
                assert f in self.app.get('/test/'+p+'/').data

    def test_app_package_file(self):
        env_create_packages(self.workspace, self.packages)

        for p in self.packages:
            for f in self.files:
                response = self.app.get('/test/'+p+'/get/'+f)
                assert response.status_code == 404

        env_create_package_files(self.workspace, self.packages, self.files)

        for p in self.packages:
            for f in self.files:
                f = '%s.tar.gz' % f
                response = self.app.get('/test/'+p+'/get/'+f)
                assert response.status_code == 200
                assert response.content_type == 'application/x-tar'
                assert response.headers.get('Content-Disposition') == \
                        'attachment; filename=%s' % f



#!/usr/bin/env python
# -*- coding: utf8 -*-

import tempfile
import unittest

from inupypi import app
from inupypi.components.packages import *
from tests.packages import *
from unipath import Path


class Test_Packages(unittest.TestCase):
    def setUp(self):
        self.app = app.test_client()
        self.workspace = Path(tempfile.mkdtemp())
        self.packages = ['p1', 'p2', 'p3', 'p4']
        self.files = ['f1', 'f2', 'f3', 'f4']
        self.files.sort(reverse=True)

        self.app.application.config['EGGBASKET_REPO'] = self.workspace

    def tearDown(self):
        self.workspace.rmtree()

    def test_get_packages_without_packages_folder(self):
        self.app.application.config['EGGBASKET_REPO'] = Path(self.workspace, 'a')

        try:
            get_packages('test')
            return False
        except:
            return True

    def test_get_packages(self):
        self.app.application.config['EGGBASKET_REPO'] = self.workspace
        env_create_packages(self.workspace, self.packages)
        env_create_package_files(self.workspace, self.packages, self.files)
        created_packages = [Path(self.workspace, 'test', package)
                for package in self.packages]
        packages = [p.filepath for p in get_packages('test')]
        assert created_packages == packages

    def test_get_packages_from_folders_only(self):
        self.app.application.config['EGGBASKET_REPO'] = self.workspace
        env_create_packages(self.workspace, self.packages)
        env_create_package_files(self.workspace, self.packages, self.files)

        packages_and_files = sorted([Path(package)
                for package in self.packages + self.files])

        created_packages = [Path(self.workspace, 'test', package)
                for package in self.packages]
        packages = [p.filepath for p in get_packages('test')]

        assert get_packages('test') != packages_and_files
        assert created_packages == packages

    def test_get_package_files(self):
        self.app.application.config['EGGBASKET_REPO'] = self.workspace
        env_create_packages(self.workspace, self.packages)

        for p in self.packages:
            assert get_package_files('test', p) == []

        env_create_package_files(self.workspace, self.packages, self.files)

        for p in self.packages:
            files = sorted([Path(self.workspace, 'test', p, '%s.tar.gz' % f)
                    for f in self.files])
            package_files = sorted([pkg.filepath
                for pkg in get_package_files('test', p)])
            assert package_files == files

    def test_get_current_package(self):
        self.app.application.config['EGGBASKET_REPO'] = self.workspace
        env_create_packages(self.workspace, self.packages)
        env_create_package_files(self.workspace, self.packages, self.files)

        for p in self.packages:
            created_packages = Path(self.workspace, 'test', p, '%s.tar.gz' %
                    sorted(self.files, reverse=True)[0])
            assert get_current_package('test', p) == created_packages

    def test_get_file(self):
        self.app.application.config['EGGBASKET_REPO'] = self.workspace
        env_create_packages(self.workspace, self.packages)
        env_create_package_files(self.workspace, self.packages, self.files)
        for p in self.packages:
            for f in self.files:
                assert get_file('test', p, f) == False

        for p in self.packages:
            for f in self.files:
                f = '%s.tar.gz' % f
                assert get_file('test', p, f) == Path(self.workspace, 'test', p, f)



#!/usr/bin/env python
# -*- coding: utf8 -*-

import tempfile

from inupypi import app, render_template, request
from inupypi.components.packages.manage import upload_package
from unipath import Path
from werkzeug import secure_filename


@app.route('/inupypi/manage/')
def manage():
    return render_template('manage/index.html')


@app.route('/inupypi/manage/package/add/', methods=['GET', 'POST'])
def package_add():
    if request.method == 'POST':
        uploaded_file = request.files['package']

        if uploaded_file:
            upload_package(uploaded_file)
    return render_template('manage/add_package.html')


@app.route('/inupypi/manage/package/remove/')
def package_del():
    return render_template('manage/remove_package.html')


@app.route('/inupypi/manage/package/update/')
def package_update():
    return render_template('manage/update_package.html')



#!/usr/bin/env python
# -*- coding: utf8 -*-

import os
from flask import Flask
from flask import abort, render_template, request, send_from_directory
from flask.ext.assets import Environment
from unipath import Path

app = Flask(__name__)

app.config.from_envvar('INUPYPI_SETTINGS')
app.template_folder = Path(Path(__file__).parent, 'templates/default')
app.config['EGGBASKET_REPO'] = app.config['EGGBASKET_REPO']

assets = Environment(app)
assets.register('css', Path(app.template_folder, 'static/style.css'),
        output='inupypi.css', filters='cssmin')

import inupypi.views
import inupypi.views.manage




#!/usr/bin/env python
# -*- coding: utf8 -*-

from inupypi import abort, app, render_template, send_from_directory
from inupypi.components.packages import *


@app.errorhandler(404)
def page_not_found(error):
    return render_template('404.html'), 404


@app.errorhandler(500)
def internal_server_error(error):
    return render_template('500.html', error=error), 500

@app.route('/')
def index():
    return render_template('index.html', eggbaskets=get_eggbaskets())

@app.route('/<eggbasket>/')
def eggbasket(eggbasket):
    return render_template('eggbasket.html', packages=get_packages(eggbasket))


@app.route('/<eggbasket>/<package_name>/')
def package(eggbasket, package_name):
    return render_template('package.html',
            files=get_package_files(eggbasket, package_name))


@app.route('/package/metadata/<eggbasket>/<package>/<filename>/')
def package_metadata(eggbasket, package, filename):
    return render_template('metadata.html',
            metadata=get_metadata(eggbasket, package, filename))


@app.route('/<eggbasket>/<package_name>/get/<filename>')
def package_get(eggbasket, package_name, filename):
    package = get_file(eggbasket, package_name, filename)

    
    if package:
        return send_from_directory(package.parent, package.name,
                as_attachment=True)
    abort(404)



#!/usr/bin/env python
# -*- coding: utf8 -*-

#import inupypi
#import tempfile
#import unittest
#
#from tests.packages import *
#from unipath import Path
#
#
#class Test_Manage(unittest.TestCase):
#    def setUp(self):
#        self.app = inupypi.app.test_client()
#
#    def tearDown(self):
#        pass
#
#    def test_manage_main_page(self):
#        assert self.app.get('/inupypi/manage/').status_code == 200
#        assert 'inetutils PyPI Server' in self.app.get('/inupypi/manage/').data



#!/usr/bin/env python
# -*- coding: utf8 -*-

import pkgtools.pkg
import tempfile

from inupypi import abort, app
from inupypi.components.packages import get_package_path
from unipath import Path
from werkzeug import secure_filename

def upload_package(uploaded_file):
    filename = secure_filename(uploaded_file.filename)
    workspace = Path(tempfile.mkdtemp())
    temp_save = Path(workspace, filename)

    uploaded_file.save(temp_save)
    package = pkgtools.pkg.SDist(temp_save)
    package_dir = Path(get_package_path(), package.name)
    package_dir.mkdir()
    temp_save.move(Path(package_dir, filename))

    workspace.rmtree()



#!/usr/bin/env python
# -*- coding: utf8 -*-

import os

from setuptools import setup, find_packages

setup(
        name='inupypi',
        version='0.2.5.2',
        packages=find_packages(),
        long_description=open("README.rst").read(),
        maintainer='Hallaj Subery',
        maintainer_email='hallajs@gmail.com',
        license="BSD",
        description='A PyPiServer based on the Flask Framework',
        platform='Linux,FreeBSD',
        include_package_data=True,
        zip_safe=False,
        test_suite='tests',
        install_requires=['Flask', 'Flask-Assets', 'cssmin', 'pkgtools',
            'unipath']
        )



#!/usr/bin/env python
# -*- coding: utf8 -*-

import os
import tarfile

from unipath import Path

def env_create_packages(workspace, packages, files=False):
    for p in packages:
        if not files:
            Path(workspace, 'test', p).mkdir(parents=True)
        else:
            Path(workspace, 'test', p).write_file('')

def env_create_package_files(workspace, packages, files):
    for p in packages:
        if files:
            for f in files:
                f = Path(workspace, 'test', p, f)
                f.mkdir(parents=True)
                version = '.'.join([x for x in f.name if x.isdigit()])
                i = env_create_package_pkginfo(p, version)
                Path(f, '%s.egg-info' % f.name).mkdir()
                Path(f, 'PKG-INFO').write_file(i)
                Path(f, '%s.egg-info' % f.name, 'PKG-INFO').write_file(i)

                os.chdir(f.parent)
                tar = tarfile.open('%s.tar.gz' % f, 'w:gz')
                tar.add('%s' % f.name)
                tar.close()
                f.rmtree()
        else:
            Path(workspace, 'test', p).mkdir(parents=True)

def env_create_package_pkginfo(name, version):
    c = []
    c.append('Metadata-Version: 1.0')
    c.append('Name: %s' % name)
    c.append('Version: %s' % version)
    c.append('Summary: ')
    c.append('Home-page: ')
    c.append('Author: ')
    c.append('Author-email: ')
    c.append('License: ')
    c.append('Description: ')
    c.append('Platform: ')
    c.append('Classifier:')

    i = '\r\n'.join(c)
    i = '%s\r\n' % i

    return i
